# -*- perl -*-

#
# Author: Slaven Rezic
#
# Copyright (C) 2016,2017,2026 Slaven Rezic. All rights reserved.
# This package is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# WWW:  https://github.com/eserte/app-plockf
#

package App::plockf;

use strict;
use vars qw($VERSION);
$VERSION = '0.05';

1;

__END__

=head1 NAME

App::plockf - execute a command while holding a file lock

=head1 DESCRIPTION

See L<plockf>. The module L<App::plockf> is just a placeholder for CPAN.

=head1 SEE ALSO

L<plockf>.

=cut
